# Biprobit CAR M-H.r
# Fit Bivariate Probit Model with BiCAR random effects
# USES M-H FOR UPDATING RHO
# Infile:  A.txt = Durham Adjacency Matrix 
#	         update phi.r and update phi zero.r
# Model:
#  y1,y2 = binary outcomes
#  w1,w2 = latent continuous normal
#  y1 = 1 if w1>0 and y2 = 1 if w2>0
#  w1,w2 ~ BVN(eta1,eta2,rho)
#  eta1 = x'beta1+phi1
#  eta2 = x'beta2+phi2
#  phi1,phi2 ~ bivariate CAR
# Jan 17, 2012
##############################
library(mvtnorm)
library(MCMCpack) # iwish distribution used in updating Sigma
library(msm)	# For tnorm function used s

A<-dget("C:/Brian/CAR/BiCAR/Makedata/A.txt") 		# Durham Adj Matrix
source ("C:/Brian/CAR/BiCAR/Functions/update phi.r")	# Update phi function		
source ("C:/Brian/CAR/BiCAR/Functions/update phi zero.r")	# Update phi function		

set.seed(031611)
 n<-129			# Number of block groups in Durham County
 nis<-rep(50,n) 		# Number of individuals per block; here it's balanced  
 tmp<-rbinom(n,1,.025)	# Zero blocks
 nis[tmp==1]<-0
 id<-rep(1:n,nis)		
 N<-length(id) 		# Total number of subjects

##################
# Adjacency Info #
##################
 m<-apply(A,1,sum)	# No. neighbors
 adjid<-rep(1:n,m)	# Tells which elements of A belong to which block
 adj<-rep(0,sum(m))
 for (i in 1:n)adj[adjid==i]<-which(A[i,]==1)  
  
########################
#  Spatial Effects     #
########################
 s<-.999999			     	     # Spatial smoothing parameter ~ 1 for intrinsic CAR
 cov<-matrix(c(4,4,4,16),2,2)      # Conditional Cov of phi1 and phi2 given phi_{-i} 
 Q<-diag(m)-s*A			     # NOTE: Indt random effects ==>  rho=0 & m=1 for all i
 
 covphi<-solve(Q)%x%cov			# 2n x 2n covariance of phis
 phi<-rmvnorm(1,sigma=covphi)		# 2n vector of spatial effects
 phitmp<-matrix(phi,ncol=2,byrow=T) # n x 2 matrix of spatial effects
 Phi1<-rep(phitmp[,1]-mean(phitmp[,1]),nis) # N x 1 phi1 vector -- Centered
 Phi2<-rep(phitmp[,2]-mean(phitmp[,2]),nis) # N x 1 phi2 vector 
 Phi<-cbind(phitmp[,1]-mean(phitmp[,1]),phitmp[,2]-mean(phitmp[,2]))

##############################
# Fixed Effects and Response #
##############################
  x<-sample(0:4,N,replace=T)
  X1<-cbind(rep(1,N),x)		# N x p design matrix for y1 (and w1)
  Xtmp1<-kronecker(X1,c(1,0))
  Xtmp2<-kronecker(X1,c(0,1)) 
  X<-cbind(Xtmp1,Xtmp2)   	# 2N x 2p overall design matrix for updating w's
  W<-matrix(0,N,2)		# Latent normal variables
  beta1<-c(.75,-.75)		# Fixed effects for y1			
  beta2<-c(-1,.5)			# Fixed effects for y2

  eta1<-X1%*%beta1+Phi1
  eta2<-X1%*%beta2+Phi2
  eta<-cbind(eta1,eta2)
  cov<-matrix(c(1,.5,.5,1),2,2) # Conditional Cov of w1,w2 given phis

  for (i in 1:N) W[i,]<-rmvnorm(1,eta[i,],cov)		# Latent normal vars
  y1<-ifelse(W[,1]<0,0,1)	# Observed outcome, y1
  y2<-ifelse(W[,2]<0,0,1)	# y2
 
  N1<-sum(y1==1)
  N2<-sum(y2==1)

##########
# Priors #
##########
  beta01<-beta02<-c(0,0) 	# Prior Mean of beta1 & beta2
  T01<-T02<-diag(.01,2)  	# Prior Prec of beta1 & beta2
  nu0<-3		  	 	# DF for Wishart prior on Sigma.phi
  c0<-diag(2)	  		# Scale matrix for Wishart Prior on Sigma.phi

#########
# Inits #
#########
  beta1<-beta2<-rep(0,2)
  beta<-c(beta1,beta2) # beta1 and beta2 for udpating w's
  w1<-w2<-rep(0,N)  
  eta<-matrix(0,N,2)
  rho<-0			# Corr(w1,w2)
  phimat<-matrix(0,n,2)	# n x 2 random effects
  Phi1<-Phi2<-rep(0,N)  # N x 1 Phi1 and Phi2
  nu<-nu0+n-1	 	# Shape parmaeter for IW - 1 for island
  Q<-diag(m)-A		# ICAR Q Matrix
  phibar<-matrix(0,n,2) # Mean spatial effects for updating phi
  Sigma<-Tauphi<-diag(2)# Simga is Cond'l Variance of y1,y2 given phi; 
				# Tauphi is spatial precision
  propvar<-.01		# Proposal Variance

#########
# Store #
#########
  nsim<-1000
  Beta<-matrix(0,nsim,4)	
  Rho<-Acc<-rep(0,nsim)
  Sigphi<-matrix(0,nsim,4)

#########
# Gibbs #
#########
ptm<-proc.time()
 for (i in 2:nsim) {
 
  # Update W1 from truncated normal, given W2 and rest
    eta<-matrix(X%*%beta,N,2,byrow=T)+cbind(Phi1,Phi2)
    mu12<-eta[,1]+rho*(w2-eta[,2]) 	# E(W1|W2)
    s12<-sqrt(1-rho^2)	           	# SD(W1|W2)
    w1[y1==0]<-rtnorm(N-N1,mu12[y1==0],s12,-Inf,0)
    w1[y1==1]<-rtnorm(N1,mu12[y1==1],s12,0,Inf)

  # Update W2 from truncated normal conditional on W1 and rest
    mu21<-eta[,2]+rho*(w1-eta[,1]) 	# E(W2|W1)
    s21<-sqrt(1-rho^2)	           	# SD(W2|W1)
    w2[y2==0]<-rtnorm(N-N2,mu21[y2==0],s21,-Inf,0)
    w2[y2==1]<-rtnorm(N2,mu21[y2==1],s21,0,Inf)    
    w<-c(t(cbind(w1,w2)))   		# Stacked

  # Update beta1     
    tau<-1/(1-rho^2)
    z<-w2-X1%*%beta2-Phi2
    v<-solve(T01+tau*t(X1)%*%X1)
    mu<-v%*%(T01%*%beta01+tau*t(X1)%*%(w1-rho*z-Phi1))  
    Beta[i,1:2]<-beta1<-c(rmvnorm(1,mu,v))

  # Update beta2
    z<-w1-X1%*%beta1-Phi1
    #v<-solve(T02+tau*t(X1)%*%X1)		# Note v same as for beta1 as long as T02=T01 
    mu<-v%*%(T02%*%beta02+tau*t(X1)%*%(w2-rho*z-Phi2))  
    Beta[i,3:4]<-beta2<-c(rmvnorm(1,mu,v))
    beta<-c(beta1,beta2)

  # Update Rho using Random Walk MH (Assume uniform prior) 
    rhonew<-rtnorm(1,rho,sqrt(propvar),-1,1) 
    eta1<-X1%*%beta1+Phi1
    eta2old<-X1%*%beta2+rho*z+Phi2
    eta2new<-X1%*%beta2+rhonew*z+Phi2
    lold<-dnorm(w1,eta1,1,log=T)+dnorm(w2,eta2old,sqrt(1-rho^2),log=T) 
    lnew<-dnorm(w1,eta1,1,log=T)+dnorm(w2,eta2new,sqrt(1-rhonew^2),log=T) 
    ratio<-sum(lnew-lold)+dtnorm(rho,rhonew,sqrt(propvar),-1,1,log=T)- 
           dtnorm(rhonew,rho,sqrt(propvar),-1,1,log=T)   # Include asymmetric proposal den 
    if (log(runif(1))<ratio) { 
      rho<-rhonew 
      Acc[i]<-1 
    }   
    if (mean(Acc[max(1,i-100):i])<.35) propvar<-max(propvar-.0001,.000001)
    if (mean(Acc[max(1,i-100):i])>.45) propvar<-min(propvar+.0001,.01)

    Rho[i]<-rho 
    Sigmae<-matrix(c(1,rho,rho,1),2,2)
    Taue<-solve(Sigmae)

 # Update phi1,phi2
   for (j in 1:n){
    phibar[j,]<-apply(phimat[adj[which(adjid==j)],],2,mean)
    if (nis[j]>0) phimat[j,]<-update.phi(y1=w1[id==j],y2=w2[id==j],X=X1[id==j,],phibar=phibar[j,],
				   	ni=nis[j],mi=m[j],beta=beta,Tau=Taue,Tauphi=Tauphi)
    if (nis[j]==0) phimat[j,]<-update.phizero(phibar=phibar[j,],mi=m[j],Tau=Tau,Tauphi=Tauphi)

   }

 # Center phis
   phi1c<-phimat[,1]-mean(phimat[,1])  
   phi2c<-phimat[,2]-mean(phimat[,2])  
   phimat<-cbind(phi1c,phi2c)
   Phi1<-rep(phi1c,nis)
   Phi2<-rep(phi2c,nis)

 # Update Sigma.phi
   Sigmaphi<-riwish(nu,c0+t(phimat)%*%Q%*%phimat)
   Sigphi[i,]<-c(Sigmaphi)  		# Store Sigma.phi values
   Tauphi<-solve(Sigmaphi)

 if (i%%25==0) print(i) 
 }
proc.time()-ptm

###########
# Results #
###########
apply(Beta[501:nsim,],2,mean)
mean(Rho[501:nsim])
apply(Sigphi[101:nsim,],2,mean)

###############
# Trace Plots #
###############
par(mfrow=c(3,1))
plot(1:nsim,Beta[1:nsim,1],type="l",col="lightgreen")
abline(h=mean(Beta[501:nsim,1]),col="blue")

plot(101:nsim,Rho[101:nsim],type="l",col="lightgreen")
abline(h=mean(Rho[501:nsim]),col="blue")

plot(1:nsim,Sigphi[1:nsim,3],type="l",col="lightgreen")
abline(h=mean(Sigphi[501:nsim,3]),col="blue")



